package de.hpi.fgis.dude.util;

import java.util.TimerTask;

/**
 * This task retrieves the memory usage of the current process and stores the maximum memory usage ever occurred.
 * 
 * @author ziawasch.abedjan
 * 
 */
public class MemoryCheckerTask extends TimerTask {
	private long maxMemoryUsed = 0;
	private long samplingFrequency = 0;
	private long totalMemoryUsed = 0;
	private long minMemoryUsed = 0;

	/**
	 * Gets the maximum amount of memory used by the current application.
	 * 
	 * @return Maximum amount of memory in KB.
	 */
	public long getMaxMemoryUsed() {
		return this.maxMemoryUsed / 1024;
	}

	@Override
	public void run() {
		long memory = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
		this.maxMemoryUsed = this.maxMemoryUsed > memory ? this.maxMemoryUsed : memory;
		this.minMemoryUsed = this.minMemoryUsed < memory && this.minMemoryUsed != 0 ? this.minMemoryUsed : memory;
		this.totalMemoryUsed = this.totalMemoryUsed + memory / 1024L;
		this.samplingFrequency++;
	}

	/**
	 * Gets the average amount of memory used by the current application.
	 * 
	 * @return Maximum amount of memory in KB.
	 */
	public long getAverageMemoryUsed() {
		if (this.samplingFrequency == 0) {
			return 0;
		}
		return this.totalMemoryUsed / this.samplingFrequency;
	}

	/**
	 * Gets the minimum amount of memory used by the current application.
	 * 
	 * @return Minimum amount of memory in KB.
	 */
	public long getMinMemoryUsed() {
		return this.minMemoryUsed / 1024;
	}

}
