package de.unipotsdam.hpi.cind.associationRuleMining;

import java.util.*;



import de.unipotsdam.hpi.cind.associationRuleMining.util.*;

public class Apriori {

	/*
	 * String for item "INCLUDED_TRUE" (i.e., included tuples)
	 * (must be smaller than all prefixes in 
	 * CindDetectionUsingApriori.PREFIXES)
	 */
	public static final String INCLUDED_TRUE = "1";
	
	/*
	 * String for item "INCLUDED_FALSE" (i.e., included tuples)
	 * (must be smaller than all prefixes in 
	 * CindDetectionUsingApriori.PREFIXES)
	 */
	public static final String INCLUDED_FALSE = "0";

	
	
	protected ArrayList<Basket> baskets;
	
	/**
	 * support as absolute number of tuples (i.e., without fraction to all tuples)
	 */
	protected int minSupport;
	

	protected double minValidity;
	
	protected double minTotalCompleteness;
	
	/**
	 * absolute number of tuples for threshold minTotalCompleteness
	 */
	protected int minTotalCompletenessAbsolute;
	
	/**
	 * support(INCLUDED_TRUE) as absolute number of tuples
	 */
	protected int includedTrueSupp;
	
	protected int largestItemSetSize = 2;
	
	public Apriori(Collection<Basket> baskets, int minSupport, 
			double minValidity, double minTotalCompleteness) {

		this.baskets = new ArrayList<Basket>(baskets);
		this.minSupport = minSupport;
		this.minValidity = minValidity;
		this.minTotalCompleteness = minTotalCompleteness;
		
		
		
	}

	/**
	 * 
	 * 
	 */
	public HashMap<List<String>, Integer> findFrequentItemSets() {

		RuntimeMeasures runtimes = RuntimeMeasures.getInstance();
		
		/*
		 * result, each frequent item set mapped to its frequency
		 */
		HashMap<List<String>, Integer> frequentItemSetsMap;
		
		long start1IS = System.currentTimeMillis();
		
		/*
		 * get 1 item sets and their frequency
		 */
		ArrayList<ItemSetString> frequentOneItemSets = determineOneItemSets();
		
		// save frequent 1-item-sets
		frequentItemSetsMap = 
			new HashMap<List<String>, Integer>(frequentOneItemSets.size()*3);

		for (ItemSetString currentItemSet : frequentOneItemSets) {
			frequentItemSetsMap.put(currentItemSet.getItems(), currentItemSet.getFrequency());
		}
		
		/*
		 * set minTotalCompletenessAbsolute, includedTrueSupp
		 */
		this.includedTrueSupp = frequentItemSetsMap.get(Arrays.asList(INCLUDED_TRUE));
		this.minTotalCompletenessAbsolute = (int)
			(this.minTotalCompleteness * this.includedTrueSupp);

		long end1IS = System.currentTimeMillis();
		
		
		
		runtimes.setRuntimeKItemSets(0, (int)(end1IS-start1IS));
		runtimes.setNrFrequentKItemSets(0, frequentOneItemSets.size());
		
		
		/*
		 * create 2-item-sets
		 */
		long start2IS = System.currentTimeMillis();
		HashTreeNode kItemSets = determineTwoItemSets(frequentOneItemSets);
		runtimes.setNrKItemSets(1, kItemSets.size());
		
		// count item set frequencies 
		countItemSetFrequencies(kItemSets);
		
		// prune infrequent item sets
//		kItemSets.pruneInfrequentItemSets(minSupport);
		kItemSets = filterFrequentTotallyCompleteItemSets(kItemSets);
		
		
		// save frequent 2-item-sets
		for (ItemSetString currentItemSet : kItemSets) {
			frequentItemSetsMap.put(currentItemSet.getItems(), currentItemSet.getFrequency());
		}
		long end2IS = System.currentTimeMillis();
		
		runtimes.setRuntimeKItemSets(1, (int)(end2IS-start2IS));
		runtimes.setNrFrequentKItemSets(1, kItemSets.size());
		
		/*
		 * create k-item-sets
		 */
		for (int itemSetSize = 2; !kItemSets.isEmpty(); itemSetSize++) {
			
			long startAprioriGen = System.currentTimeMillis();
			
			// create item set candidates
			kItemSets = aprioriGen(kItemSets, itemSetSize);
			runtimes.setNrKItemSets(itemSetSize, kItemSets.size());
			
			// count item set frequencies 
			countItemSetFrequencies(kItemSets);
			
			// prune infrequent item sets
//			kItemSets.pruneInfrequentItemSets(minSupport);
			kItemSets = filterFrequentTotallyCompleteItemSets(kItemSets);
			
			// save frequent k-item-sets
			for (ItemSetString currentItemSet : kItemSets) {
				frequentItemSetsMap.put(currentItemSet.getItems(), currentItemSet.getFrequency());
			}
			
			long endKItemSets = System.currentTimeMillis();
			
			runtimes.setRuntimeKItemSets(itemSetSize, (int)(endKItemSets-startAprioriGen));
			runtimes.setNrFrequentKItemSets(itemSetSize, kItemSets.size());
			
			this.largestItemSetSize = itemSetSize + 1;
			
		}

		
//		System.out.println("number of included groups: " + baskets.size());
//		System.out.println("minFrequency: " + minSupport);
//		System.out.println("minCompleteness: " + minTotalCompletenessAbsolute);
		
		return frequentItemSetsMap;

	}

	/**
	 * Count item set frequencies of specified item sets and remove baskets that
	 * contain no frequent item set
	 * 
	 * @param kItemSets
	 * @return true, if at least one basket was deleted (because it contained no
	 *         frequent item set)
	 */
	protected boolean countItemSetFrequencies(HashTreeNode kItemSets) {
		
		boolean deletedBasket = false;
		for (Iterator<Basket> it = baskets.iterator(); it.hasNext();) {
			Basket currentBasket = it.next();
			
			boolean containsFrequentItemSet = 
				kItemSets.increaseSupportOfContainedItemSets(currentBasket);
			if ( !containsFrequentItemSet ) {
				it.remove();
				deletedBasket = true;
			}
		}
		
		return deletedBasket;
	}
	
	protected HashTreeNode filterFrequentTotallyCompleteItemSets(HashTreeNode root) {
		
		int splitSize = Math.min(HashTreeNode.DEFAULT_SPLIT_THRESHOLD, root.size()/3);
		HashTreeNode frequentItemSets = new HashTreeNode(splitSize);
//		System.err.println("splitSize (Apriori.filterFrequent): " + splitSize);
		
		for (ItemSetString currentItemSet : root) {
			if (currentItemSet.getFrequency() >= minSupport 
					&& currentItemSet.getFrequency() >= minTotalCompletenessAbsolute 
					) {
				frequentItemSets.addItemSet(currentItemSet);
			}
		}
		
		return frequentItemSets;
	}
	
	
	
	/**
	 * 
	 * @param frequentItemSets
	 */
	public ArrayList<AssociationRule> generateAssociationRulesWithRhsTrue(
			HashMap<List<String>, Integer> frequentItemSets) {
		
		
		ArrayList<AssociationRule> result = 
			new ArrayList<AssociationRule>(frequentItemSets.size());
		
		/*
		 * confidence = validity = support(X \cup INCLUDED_TRUE) / support(X)
		 * confidenceBar = totalCompleteness = support(X \cup INCLUDED_TRUE) / support(INCLUDED_TRUE)
		 */
		
		// support(INCLUDED_TRUE)
//		int includedTrueSupp = frequentItemSets.get(Arrays.asList(Apriori.INCLUDED_TRUE));
		// support(X \cup INCLUDED_TRUE)
		int lhsRhsSupp;
		// support(X)
		int lhsSupp ;
		
		double validity, totalCompleteness;
		
		for (List<String> currentItemSet : frequentItemSets.keySet()) {
			
    		if (currentItemSet.size() == 1) 
    			continue;
			
			if (currentItemSet.get(0).equals(INCLUDED_TRUE)) {
				
				lhsRhsSupp = frequentItemSets.get(currentItemSet);
				
				List<String> lhs = currentItemSet.subList(1, currentItemSet.size());
				lhsSupp = frequentItemSets.get(lhs);
				
				validity = (double) lhsRhsSupp / lhsSupp;
				totalCompleteness = (double) lhsRhsSupp / includedTrueSupp; 
				
				if (validity > minValidity && totalCompleteness > minTotalCompleteness) {
					result.add(new AssociationRule(
							lhs, lhsRhsSupp, lhsSupp, validity, totalCompleteness));
				}
				
			}
			
		}
		
		return result;
	}
	
	
	protected ArrayList<ItemSetString> determineOneItemSets() {

		/*
		 *  get all 1-item-sets and their frequency
		 */
		HashMap<String, ItemSetString> oneItemSetsMap = 
			new HashMap<String, ItemSetString>( baskets.size() * 5 );

		ItemSetString currentItemSet;
		for (Basket currentBasket : baskets) {
			for (String basketItem : currentBasket) {

				currentItemSet = oneItemSetsMap.get(basketItem);
				if ( currentItemSet == null ) {
					currentItemSet = new ItemSetString(basketItem);
					oneItemSetsMap.put(basketItem, currentItemSet);
				}
				currentItemSet.increaseFrequency();
				
			}
		}

		RuntimeMeasures runtimes = RuntimeMeasures.getInstance();
		runtimes.setNrKItemSets(0, oneItemSetsMap.size());
		
		ArrayList<ItemSetString> oneItemSets = 
			new ArrayList<ItemSetString>(oneItemSetsMap.values());
		ArrayList<ItemSetString> frequentOneItemSets = 
			new ArrayList<ItemSetString>(oneItemSetsMap.values().size());

		/*
		 *  filter infrequent 1-item-sets
		 */
		for (ItemSetString itemSet : oneItemSets) {
			if ( itemSet.getFrequency() >= minSupport ) 
				frequentOneItemSets.add(itemSet);
		}
		
		
		return frequentOneItemSets;

	}
	
	/**
	 * Build two item sets by combining each frequent one item set with all
	 * other frequent one item sets. Remove infrequent item sets.
	 * 
	 * @return frequent two item sets 
	 */
	protected HashTreeNode determineTwoItemSets(ArrayList<ItemSetString> frequentOneItemSets) {
		
		int splitSize = Math.min(HashTreeNode.DEFAULT_SPLIT_THRESHOLD, frequentOneItemSets.size()/3);
		HashTreeNode twoItemSets = new HashTreeNode(splitSize);
//		System.err.println("splitSize (Apriori.twoItemSets): " + splitSize);
		
		/*
		 * Combine each frequent one item set with all other frequent one item
		 * sets.
		 * 
		 * Consider order of items.
		 */
		Collections.sort(frequentOneItemSets);
		for (int i = 0; i < frequentOneItemSets.size(); i++) {
			for (int j = i+1; j < frequentOneItemSets.size(); j++) {

				twoItemSets.addItemSet(new ItemSetString(
						frequentOneItemSets.get(i).getItem(0), 
						frequentOneItemSets.get(j).getItem(0)));
				
			}
		}
		
		return twoItemSets;
		
	}
	
	

	/**
	 * Builds new candidate item sets at level <code>this.level+1</code> from
	 * given frequent candidate item sets.
	 * 
	 * 
	 * Traverses a hash tree with {@link ItemSetString} objects (of equal size)
	 * and creates all candidate item sets with size
	 * <code>ItemSetString.size()+1</code> .
	 * 
	 * A new candidate results from two ItemsSetString objects <code>p</code>
	 * and <code>q</code> of size <code>k</code> with
	 * <code>p.item_1=q.item_1, ..., p.item_k-1 = q.item_k-1</code> and
	 * <code>p.item_k < q.item_k</code> as new ItemSetString
	 * <code>p.item_1, ..., p.item_k, q.item_k</code>.
	 * 
	 * 
	 * @return
	 */
	protected HashTreeNode aprioriGen(HashTreeNode frequentItemSets, int itemSetSize) {

		List<String> itemSetPrefix1, itemSetPrefix2;
		
		/*
		 * get frequentItemSets as sorted list to leverage that equal prefixed
		 * item sets lay next to each other, i.e., there will be no two equal
		 * item set prefixes (with current first item set) after first two
		 * unequal item set prefixes
		 */
		List<ItemSetString> sortedFrequentItemSets = frequentItemSets.toSortedList();

		int splitSize = Math.min(
				HashTreeNode.DEFAULT_SPLIT_THRESHOLD, sortedFrequentItemSets.size()/itemSetSize);
		HashTreeNode candidates = new HashTreeNode(splitSize);
//		System.err.println("      splitSize (Apriori.aprioriGen): " + splitSize);
		
		
		/*
		 * compare item set prefixes pairwise and 
		 * prune candidates with infrequent subsets
		 */
		for (int i = 0; i < sortedFrequentItemSets.size(); i++) {
			for (int j = i+1; j < sortedFrequentItemSets.size(); j++) {

				itemSetPrefix1 = sortedFrequentItemSets.get(i).
						getItems().subList(0, itemSetSize - 1);
				itemSetPrefix2 = sortedFrequentItemSets.get(j).
						getItems().subList(0, itemSetSize - 1);

				// equal prefixes -> new candidate
				if ( itemSetPrefix1.equals(itemSetPrefix2) ) {
					
					ItemSetString candidate = new ItemSetString(
							sortedFrequentItemSets.get(i).getItems(), 
							sortedFrequentItemSets.get(j).getItem(itemSetSize-1));
					
					if ( allSubSetsFrequent(frequentItemSets, itemSetSize, candidate))
						candidates.addItemSet(candidate);
				} 
				// different prefixes -> no new candidate can follow starting with itemSetPrefix1 
				else {
					break;
				}

			}
		}
		
		return candidates;
	}
	

	/**
	 * Test all sub item sets except i_1,...,i_k-1 (= itemSetPrefix1) and
	 * i_1,...,i_k-2,i_k (= itemSetPrefix2)
	 * 
	 * and
	 * 
	 * look for sub item sets in frequent item sets and omit candidate if sub
	 * item set is not contained
	 * 
	 * @param frequentItemSets hash tree with item sets of size itemSetSize
	 * @param frequentItemSetSize
	 * @param candidate item set of size frequentItemSetSize+1
	 * 
	 * @return true, if all item sub sets are frequent
	 * 
	 */
	protected boolean allSubSetsFrequent(
			HashTreeNode frequentItemSets, int frequentItemSetSize, ItemSetString candidate) {
		
		List<String> completeItemSet = candidate.getItems();
		ItemSetString currentSubItemSet;
		for (int i = 0; i < frequentItemSetSize-1; i++) {
			// get sub item set
			currentSubItemSet = new ItemSetString(
					completeItemSet.subList(0, i), 
					completeItemSet.subList(i+1, frequentItemSetSize+1));

			// test its frequency
			if ( !frequentItemSets.contains(currentSubItemSet) ) {
				// found infrequent sub set
				return false;
			}
		}
		
		// all sub item sets contained -> valid candidate
		return true;
	}

}
