package de.unipotsdam.hpi.cind.associationRuleMining;

import java.sql.*;
import java.util.*;

import de.unipotsdam.hpi.cind.associationRuleMining.util.*;

public class CoveringCindDetectionUsingApriori extends
		CindDetectionUsingApriori {

	public CoveringCindDetectionUsingApriori(Connection conn) {
		super(conn);
	}

	/**
	 * Executes specified query and reads data into baskets. Each value is
	 * preceded by a prefix to preserve column affiliation.
	 * 
	 * 
	 * @param query
	 *            Query to get outer join over depending and referenced
	 *            relations. First attribute is referencing column indicating
	 *            groups, second attribute is referenced column indicating
	 *            inclusion (NOT NULL) or not (NULL).
	 * @param prefixes
	 *            Prefixes for dependent columns, i.e., column
	 *            2,...,prefixes.length+1
	 * @return
	 * @throws SQLException
	 */
	@Override
	protected ArrayList<Basket> readBaskets(
			String query, String[] prefixes) 
			throws SQLException {
		
		long start = System.currentTimeMillis();
		
		
		// get data as left outer join 
		Statement stmt = conn.createStatement();;
		ResultSet rsBaskets = stmt.executeQuery(query);
		
		// store data to baskets with
		// - referenced column 0 or 1 
		// - values of first column prefixed with 1|, second column with 2|, ...
		// - values of same group in one basket (i.e. several values with same prefix)
		ArrayList<Basket> baskets = new ArrayList<Basket>(1024);
		HashSet<String> currentBasket = new HashSet<String>();
		Object currentItem;
		
		String lastGroupID = null, currentGroupID;
		Object currentGroupIdObject;
		
		while( rsBaskets.next() ) {
			
			currentGroupIdObject = rsBaskets.getObject(1);
			if (currentGroupIdObject == null) { 
				continue;
			} else {
				currentGroupID = currentGroupIdObject.toString();
			}
			
			
			/*
			 *  start of new group (i.e. new basket)
			 */
			if ( !currentGroupID.equals(lastGroupID) ) {
				// basket to list of baskets
				baskets.add( new Basket(currentBasket) );
				currentBasket.clear();
				
				// handle INCLUDED_TRUE, INCLUDED_FALSE
				if (rsBaskets.getObject(2) == null)
					currentBasket.add(Apriori.INCLUDED_FALSE);
				else
					currentBasket.add(Apriori.INCLUDED_TRUE);
				
				// update lastGroupID
				lastGroupID = currentGroupID;
			}
			
			// handle other attributes
			for (int i = 3; i <= prefixes.length+2; i++) {
				
				currentItem = rsBaskets.getObject(i);
				if (currentItem == null )
					continue;
				
				currentBasket.add(prefixes[i-3] + currentItem.toString());
			}
			
			
			
		}
		
		rsBaskets.close();
		stmt.close();
		
		long end = System.currentTimeMillis();
		
		RuntimeMeasures runtimes = RuntimeMeasures.getInstance();
		runtimes.setRuntimeReadData( (int)(end-start) );

//		for (Basket basket : baskets) {
//			System.out.println(basket.toString());
//		}
		System.out.println("nr of baskets: " + baskets.size());
		
		return baskets;
	}
	
	/**
	 * Executes specified query and reads data into baskets. Each value is
	 * preceded by a prefix to preserve column affiliation.
	 * 
	 * 
	 * @param query
	 *            Query to get outer join over depending and referenced
	 *            relations. First attribute is referencing column indicating
	 *            groups, second attribute is referenced column indicating
	 *            inclusion (NOT NULL) or not (NULL).
	 * @param prefixes
	 *            Prefixes for dependent columns, i.e., column
	 *            2,...,prefixes.length+1
	 * @return
	 * @throws SQLException
	 */
	@Override
	protected List<ArrayList<Basket>> readBasketsConstrained(
			String query, String[] prefixes) 
			throws SQLException {
		
		long start = System.currentTimeMillis();
		
//		System.err.println(Arrays.toString(prefixes));
		
		// get data as left outer join 
		Statement stmt = conn.createStatement();;
		ResultSet rsBaskets = stmt.executeQuery(query);
		
		// store data to baskets with
		// - referenced column 0 or 1 
		// - values of first column prefixed with 1|, second column with 2|, ...
		// - values of same group in one basket (i.e. several values with same prefix)
		ArrayList<Basket> includedBaskets = new ArrayList<Basket>(1024);
		ArrayList<Basket> nonIncludedBaskets = new ArrayList<Basket>(1024);
		boolean isIncluded = false;
		
		HashSet<String> currentBasket = new HashSet<String>();
		Object currentItem;
		
		String lastGroupID = null, currentGroupID;
		Object currentGroupIdObject;
		
		while( rsBaskets.next() ) {
			
			currentGroupIdObject = rsBaskets.getObject(1);
			if (currentGroupIdObject == null) { 
				continue;
			} else {
				currentGroupID = currentGroupIdObject.toString();
			}
			
			
			/*
			 *  start of new group (i.e. new basket)
			 */
			if ( !currentGroupID.equals(lastGroupID) ) {
				// basket to list of baskets
				if (isIncluded) {
					includedBaskets.add( new Basket(currentBasket) );
				} else {
					nonIncludedBaskets.add( new Basket(currentBasket) );
				}
				currentBasket.clear();
				
				// handle INCLUDED_TRUE, INCLUDED_FALSE
				if (rsBaskets.getObject(2) == null) {
					currentBasket.add(Apriori.INCLUDED_FALSE);
					isIncluded = false;
				} else {
					currentBasket.add(Apriori.INCLUDED_TRUE);
					isIncluded = true;
				}
				
				// update lastGroupID
				lastGroupID = currentGroupID;
			}
			
			// handle other attributes
			for (int i = 3; i <= prefixes.length+2; i++) {
				
				currentItem = rsBaskets.getObject(i);
				if (currentItem == null )
					continue;
				
				currentBasket.add(prefixes[i-3] + currentItem.toString());
			}
			
//			// handle other attributes, except last one (expected to be column description)
//			for (int i = 3; i <= prefixes.length+1; i++) {
//				
//				currentItem = rsBaskets.getObject(i);
//				if (currentItem == null )
//					continue;
//				
//				currentBasket.add(prefixes[i-3] + currentItem.toString());
//			}
//			
//			// handle last attribute (= description)
//			// split it up at comma and store each part into basket
//			currentItem = rsBaskets.getObject(prefixes.length+2);
//			if (currentItem == null )
//				continue;
//
//			String description = currentItem.toString();
//			String[] descriptionItems = description.substring(1, description.length()-4).split("[;,] ");
//			for (int i = 0; i < descriptionItems.length; i++) {
//				currentBasket.add(prefixes[prefixes.length-1] + descriptionItems[i]);
//			}
			
			
		}
		
		rsBaskets.close();
		stmt.close();
		
		long end = System.currentTimeMillis();
		
		RuntimeMeasures runtimes = RuntimeMeasures.getInstance();
		runtimes.setRuntimeReadData( (int)(end-start) );

		ArrayList<ArrayList<Basket>> baskets = new ArrayList<ArrayList<Basket>>();
		baskets.add(includedBaskets);
		baskets.add(nonIncludedBaskets);
		
		return baskets;
	}
	
	
}
