package de.unipotsdam.hpi.cind.associationRuleMining.statistics;

import java.io.*;
import java.util.*;

import samples.preview_new_graphdraw.transform.*;

import de.unipotsdam.hpi.cind.associationRuleMining.util.*;

public class FilterConditions {

	
	
	
	public static void main(String[] args) throws IOException  {

		String file = args[0];
		String outputFile = args[1];
		String outputFileRemoved = args[2];
		
		BufferedReader reader = new BufferedReader( new FileReader(file));
		BufferedWriter writer = new BufferedWriter( new FileWriter(outputFile) );
		BufferedWriter writerRemoved = new BufferedWriter( new FileWriter(outputFileRemoved) );
		
		String nextLine = reader.readLine();
		if (nextLine == null) 
			throw new RuntimeException("file " + file + " is empty");

		/*
		 * read lines and write filtered CINDs to outputFile, and removed CINDs to outputFileRemoved
		 */
		ConditionalIND currentCIND;
		int nrCINDs = 0;
		int nrFilteredCINDs = 0;
		while (nextLine != null) {
			currentCIND = FilterConditions.readCondition(nextLine);
			nrCINDs++;
			if ( FilterConditions.filterCIND(currentCIND) ) {
				writer.write(String.valueOf(currentCIND.getConditionsAttributes().size()));
				writer.write("; ");
				writer.write(currentCIND.toStringGeneral());
				writer.write("\n");
				nrFilteredCINDs++;
			} else {
				writerRemoved.write(String.valueOf(currentCIND.getConditionsAttributes().size()));
				writerRemoved.write("; ");
				writerRemoved.write(currentCIND.toStringGeneral());
				writerRemoved.write("\n");
			}
			
			nextLine = reader.readLine();
        }
		
        reader.close();
		
		
		writer.flush();
        writer.close();
		
        writerRemoved.flush();
        writerRemoved.close();
		
		System.out.println("Nr CINDs: " + nrCINDs);
		System.out.println("Nr filtered CINDs: " + nrFilteredCINDs);
	
	}
	
	
	/**
	 * Reads one line of cind files of form 
	 * description = "Anwalt, Politiker, Gouverneur und Senator von Connecticut"@de, ; lhsRhsSupp = 1.4239284048798026E-5, validity = 1.0, totalCompleteness = 3.0028226532940963E-5
	 * 
	 * @param file
	 * @param noLines
	 * @return
	 * @throws IOException
	 */
	public static ConditionalIND readCondition(String line) {
		
		
			/*
			 *  split line 
			 */
			String[] split = line.split(";");
			String conditions = split[0];
			String measures = split[1];
			
			
			/*
			 * parse conditions
			 */
			
			ArrayList<String> attributeNames = new ArrayList<String>();
			ArrayList<String> values = new ArrayList<String>();
			
			String[] conditionsSplitted = conditions.split(", ");
			
			
			
			for (int i = 0; i < conditionsSplitted.length; i++) {
				
				String[] attributeValueSplit = conditionsSplitted[i].split(" = ");
				
				attributeNames.add(attributeValueSplit[0]);
				String value = attributeValueSplit[1];

				// handle commas in condition values
				while ( (i+1 < conditionsSplitted.length) 
						&& !( conditionsSplitted[i+1].startsWith("name")
								|| conditionsSplitted[i+1].startsWith("givenName") 
								|| conditionsSplitted[i+1].startsWith("surname") 
								|| conditionsSplitted[i+1].startsWith("birthdate") 
								|| conditionsSplitted[i+1].startsWith("birthcentury") 
								|| conditionsSplitted[i+1].startsWith("birthyear") 
								|| conditionsSplitted[i+1].startsWith("birthplace") 
								|| conditionsSplitted[i+1].startsWith("deathdate") 
								|| conditionsSplitted[i+1].startsWith("deathcentury") 
								|| conditionsSplitted[i+1].startsWith("deathyear") 
								|| conditionsSplitted[i+1].startsWith("deathplace") 
								|| conditionsSplitted[i+1].startsWith("description")) ) {


					value = value + ", " + conditionsSplitted[i+1];
					i++;

				}
				values.add(value);
			}
			
					
			/*
			 * parse measures
			 */
			double conditionAndIncludedSupp;
			double validity;
			double totalCompleteness;
			try {
				String[] measureSplit = measures.split(",");
				String[] tempSplit = measureSplit[0].split("=");
				conditionAndIncludedSupp = Double.valueOf(tempSplit[1]);
				
				tempSplit = measureSplit[1].split("=");
				validity = Double.valueOf(tempSplit[1]);
				
				tempSplit = measureSplit[2].split("=");
				totalCompleteness = Double.valueOf(tempSplit[1]);
			} catch (ArrayIndexOutOfBoundsException e) {
				System.out.println(line);
				throw e;
			}
			
			
			
			

			
			// create and return ConditionalIND
			return new ConditionalIND(attributeNames, values, 
					conditionAndIncludedSupp, validity, totalCompleteness);
			
	}
	
	
	/**
	 * 
	 * @param cindList
	 * @return true, if cind is non-redundant (and should be preserved)
	 */
	public static boolean filterCIND(ConditionalIND conditionalIND) {
		
		ArrayList<String> attributes = conditionalIND.getConditionsAttributes();
		if (attributes.contains("birthyear") 
				&& (attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
			return false;
		}

		if (attributes.contains("deathyear") 
				&& (attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
			return false;
		}

		if (attributes.contains("birthdate") 
				&& (attributes.contains("birthyear") || attributes.contains("deathyear") 
						|| attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
			return false;
		}

		if (attributes.contains("deathdate") 
				&& (attributes.contains("birthyear") || attributes.contains("deathyear") 
						|| attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
			return false;
		}

		if (attributes.contains("birthcentury") 
				&& attributes.contains("deathcentury") ) {
			return false;
		}
		
		return true;
	}
	
	
//	public static void main(String[] args) throws IOException  {
//
//		
//		ArrayList<ConditionalIND> cinds = FilterConditions.readConditions(args[0], Integer.valueOf(args[1]) );
////				"Y:\\promotion\\aladin\\cindPerformance\\scatterDiagramms\\allCinds-kommaInValue-head.csv", 20);
//		
//		ArrayList<ConditionalIND> filteredCINDs = FilterConditions.filterCINDs(cinds);
//		
//		System.out.println("Nr CINDs: " + cinds.size());
//		System.out.println("Nr filtered CINDs: " + filteredCINDs.size());
//	
//		
//		String outputFile = args[2];
//		
//		BufferedWriter writer = new BufferedWriter( new FileWriter(outputFile) );
//		
//		for (ConditionalIND conditionalIND : filteredCINDs) {
//			writer.write(String.valueOf(conditionalIND.getConditionsAttributes().size()));
//			writer.write("; ");
//			writer.write(conditionalIND.toStringGeneral());
//			writer.write("\n");
//		}
//		writer.flush();
//        writer.close();
//		
//	}
	
	/**
	 * Reads cind files of form 
	 * description = "Anwalt, Politiker, Gouverneur und Senator von Connecticut"@de, ; lhsRhsSupp = 1.4239284048798026E-5, validity = 1.0, totalCompleteness = 3.0028226532940963E-5
	 * 
	 * @param file
	 * @param noLines
	 * @return
	 * @throws IOException
	 */
	public static ArrayList<ConditionalIND> readConditions(String file, int noLines) throws IOException {
		
//		String file = "Y:\\promotion\\aladin\\cindPerformance\\scatterDiagramms\\allCinds-kommaInValue-head.csv";
		BufferedReader reader = new BufferedReader( new FileReader(file));
		
			
		ArrayList<ConditionalIND> cindList = new ArrayList<ConditionalIND>(noLines);
		
		String nextLine = reader.readLine();
		if (nextLine == null) 
			throw new RuntimeException("file " + file + " is empty");

			
			
		while (nextLine != null) {
			
//			System.out.println(nextLine);
			
			/*
			 *  split line 
			 */
			String[] split = nextLine.split(";");
			String conditions = split[0];
			String measures = split[1];
			
			
			/*
			 * parse conditions
			 */
			
			ArrayList<String> attributeNames = new ArrayList<String>();
			ArrayList<String> values = new ArrayList<String>();
			
			String[] conditionsSplitted = conditions.split(", ");
			
			
			
			for (int i = 0; i < conditionsSplitted.length; i++) {
				
				String[] attributeValueSplit = conditionsSplitted[i].split(" = ");
				
				attributeNames.add(attributeValueSplit[0]);
				String value = attributeValueSplit[1];

				// handle commas in condition values
				while ( (i+1 < conditionsSplitted.length) 
						&& !( conditionsSplitted[i+1].startsWith("name")
								|| conditionsSplitted[i+1].startsWith("givenName") 
								|| conditionsSplitted[i+1].startsWith("surname") 
								|| conditionsSplitted[i+1].startsWith("birthdate") 
								|| conditionsSplitted[i+1].startsWith("birthcentury") 
								|| conditionsSplitted[i+1].startsWith("birthyear") 
								|| conditionsSplitted[i+1].startsWith("birthplace") 
								|| conditionsSplitted[i+1].startsWith("deathdate") 
								|| conditionsSplitted[i+1].startsWith("deathcentury") 
								|| conditionsSplitted[i+1].startsWith("deathyear") 
								|| conditionsSplitted[i+1].startsWith("deathplace") 
								|| conditionsSplitted[i+1].startsWith("description")) ) {


					value = value + ", " + conditionsSplitted[i+1];
					i++;

				}
				values.add(value);
			}
			
					
			/*
			 * parse measures
			 */
			double conditionAndIncludedSupp;
			double validity;
			double totalCompleteness;
			try {
				String[] measureSplit = measures.split(",");
				String[] tempSplit = measureSplit[0].split("=");
				conditionAndIncludedSupp = Double.valueOf(tempSplit[1]);
				
				tempSplit = measureSplit[1].split("=");
				validity = Double.valueOf(tempSplit[1]);
				
				tempSplit = measureSplit[2].split("=");
				totalCompleteness = Double.valueOf(tempSplit[1]);
			} catch (ArrayIndexOutOfBoundsException e) {
				System.out.println(nextLine);
				throw e;
			}
			
			
			
			

			
			// create ConditionalIND
			cindList.add( new ConditionalIND(attributeNames, values, 
					conditionAndIncludedSupp, validity, totalCompleteness) );
			
			nextLine = reader.readLine();
        }
		
        reader.close();
        
		return cindList;
	}
	

	
	
	public static ArrayList<ConditionalIND> filterCINDs(ArrayList<ConditionalIND> cindList) {
		
		ArrayList<ConditionalIND> filteredCINDs = new ArrayList<ConditionalIND>(cindList.size());
		
		// name givenName surname birthdate birthcentury birthyear birthplace deathdate deathcentury deathyear deathplace description
		
		for (ConditionalIND conditionalIND : cindList) {
			
			ArrayList<String> attributes = conditionalIND.getConditionsAttributes();
			if (attributes.contains("birthyear") 
					&& (attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
				continue;
			}
			
			if (attributes.contains("deathyear") 
					&& (attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
				continue;
			}
			
			if (attributes.contains("birthdate") 
					&& (attributes.contains("birthyear") || attributes.contains("deathyear") 
							|| attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
				continue;
			}
			
			if (attributes.contains("deathdate") 
					&& (attributes.contains("birthyear") || attributes.contains("deathyear") 
							|| attributes.contains("birthcentury") || attributes.contains("deathcentury"))) {
				continue;
			}
			
			if (attributes.contains("birthcentury") 
					&& attributes.contains("deathcentury") ) {
				continue;
			}
			
			
			filteredCINDs.add(conditionalIND);
			
		}
		
		
		return filteredCINDs;
	}
	
	
	
}
