package de.unipotsdam.hpi.cind.associationRuleMining.util;

import java.util.*;

/**
 * Represent one basket / transaction.
 * 
 * @author jana.bauckmann
 *
 */
public class Basket implements Iterable<String> {

	/**
	 *  sorted list of items in this basket
	 */
	private ArrayList<String> items;
	
	/**
	 * Create new basket containing the given items in given order.
	 * 
	 */
	public Basket(List<String> items) {
		this.items = new ArrayList<String>(items);
	}
	
	/**
	 * Create new basket containing the given items in given order.
	 * 
	 */
	public Basket(String... items) {
		this(Arrays.asList(items));
	}
	
	
	public Basket(HashSet<String> items) {
		this.items = new ArrayList<String>(items);
		Collections.sort(this.items);
	}

	/**
	 * Tests if suffix of this basket contains suffix of of specified item set.
	 * 
	 * @param itemSet
	 * @param startIndexItemSet
	 *            index of first item to start inclusion test
	 * @param startIndexBasket
	 *            index of first basket item to start inclusion test
	 * @return true, if basket contains all items of <code>itemSet</code>
	 *         starting from <code>startIndexItemSet</code> and
	 *         <code>startIndexBasket</code>
	 */
	public boolean contains(ItemSetString itemSet, int startIndexItemSet, int startIndexBasket) {
		
		Iterator<String> basketIt = this.items.listIterator(startIndexBasket);
		Iterator<String> setIt = itemSet.listIterator(startIndexItemSet); 
		String basketItem;
		String setItem;
		
		// search for all setItems in basketItems
		// leverage sorted item sets 
		while (setIt.hasNext()) {
			setItem = (String) setIt.next();
		
			do {
				if (basketIt.hasNext()) 
					basketItem = (String) basketIt.next();
				else
					return false;
			} while ( !setItem.equals(basketItem) );
		}

		// found all setItems
		return true;
		
	}

	@Override
	public Iterator<String> iterator() {
		return items.iterator();
	}

	/**
	 * Returns a list iterator of the elements in this basket (in proper
	 * sequence), starting at the specified position. The specified
	 * index indicates the first element that would be returned by an initial
	 * call to next. An initial call to previous would return the element with
	 * the specified index minus one.
	 * 
	 * @param index
	 *            index of first element to be returned from the list iterator
	 *            (by a call to the next method)
	 * 
	 * @return a list iterator of the elements in this list (in proper
	 *         sequence), starting at the specified position in this list
	 *         
	 * 
	 */
	public ListIterator<String> listIterator(int index) {
		return items.listIterator(index);
	}
	
	/**
	 * 
	 * @param index
	 * @return Item at specified index. 
	 */
	public String getItem(int index) {
		return items.get(index);
	}
	
	/**
	 * 
	 * @return size of this basket, i.e., number of items
	 */
	public int size() {
		return items.size();
	}
	
	
	public String toString() {
		return items.toString();
	}
	
}
