package de.unipotsdam.hpi.cind.associationRuleMining.util;

import java.util.*;


public class ConditionalIND {

	/**
	 * key: attribute name, value: attribute value
	 */
//	private HashMap<String, String> conditions;
	
	/**
	 * attribute names for conditions
	 * Note: index corresponds to index of {@link conditionsValues}.
	 */
	private ArrayList<String> conditionsAttributes;
	
	/**
	 * attribute values for conditions 
	 * Note: index corresponds to index of {@link conditionsAttributes}.
	 */
	private ArrayList<String> conditionsValues;

	
	
	// support(X \cup INCLUDED_TRUE)
	private int conditionAndIncludedSupp;
	private double conditionAndIncludedSuppRelative;
	// support(X)
	private int conditionSupp ;
	
	private int relationCardinality;
	
	private double validity, totalCompleteness;

	/**
	 * 
	 * @param rule
	 * @param attributeNames
	 */
	public ConditionalIND(AssociationRule rule,
			HashMap<String, String> attributeNames, 
			int prefixSize,
			int relationCardinality) {

//		conditions = new HashMap<String, String>(rule.size());
		conditionsAttributes = new ArrayList<String>(rule.size());
		conditionsValues = new ArrayList<String>(rule.size());
		
		
		for (String currentCondition : rule) {
			String prefix = currentCondition.substring(0, prefixSize);
			String conditionValue = currentCondition.substring(prefixSize);
			
			conditionsAttributes.add(attributeNames.get(prefix));
			conditionsValues.add(conditionValue);
		}
		
		this.conditionAndIncludedSupp = rule.getLhsRhsSupp();
		this.conditionSupp = rule.getLhsSupp();
		this.relationCardinality = relationCardinality;
		this.validity = rule.getValidity();
		this.totalCompleteness = rule.getTotalCompleteness();
	}
	
	
	public ConditionalIND(ArrayList<String> attributes, ArrayList<String> values, 
			double conditionAndIncludedSupp, double validity, double totalCompleteness) {
		
		this.conditionsAttributes = attributes;
		this.conditionsValues = values;
		this.conditionAndIncludedSuppRelative = conditionAndIncludedSupp;
		this.validity = validity;
		this.totalCompleteness = totalCompleteness;
	}
	
	
	public ArrayList<String> getConditionsAttributes() {
		return conditionsAttributes;
	}


	public String toString() {
		
		StringBuilder sb = new StringBuilder();
		
		for (int i = 0; i < conditionsAttributes.size(); i++) {
			sb.append(conditionsAttributes.get(i));
			sb.append(" = ");
			sb.append(conditionsValues.get(i));
			sb.append(", ");
		}
		
//		sb.append(conditions);
		sb.append("; lhsRhsSupp = ").append(conditionAndIncludedSupp)
			.append("(").append((double) conditionAndIncludedSupp/relationCardinality).append(")") ;
		sb.append("; lhsSupp = ").append(conditionSupp)
			.append("(").append((double) conditionSupp/relationCardinality).append(")") ;
		sb.append("; validity = ").append(validity);
		sb.append("; totalCompleteness = ").append(totalCompleteness);
		
		return sb.toString();
	}
	
	public String toStringGeneral() {
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < conditionsAttributes.size(); i++) {
			sb.append(conditionsAttributes.get(i)).append(" = ").append(conditionsValues.get(i)).append(", ");
		}
		
		sb.append("; lhsRhsSupp = ").append(conditionAndIncludedSuppRelative);
		sb.append("; validity = ").append(validity);
		sb.append("; totalCompleteness = ").append(totalCompleteness);
		
		return sb.toString();
	}
	
}
