package de.unipotsdam.hpi.cind.associationRuleMining.util;

import java.util.*;


public class RuntimeMeasures {

	private static RuntimeMeasures instance = new RuntimeMeasures();
	
	private double minSupport = 0; 
	private double minValidity = 0; 
	private double minTotalCompleteness = 0;
	
	/**
	 * time to read data [ms]
	 */
	private int runtimeReadData = 0;
	
	/**
	 * times to generate k-item-sets [ms]
	 */
	private Integer[] runtimeKItemSets;
	
	/**
	 * time to generate rules
	 */
	private Integer runtimeRuleGen = 0;
	
	private Integer runtimeOverAll = 0;
	
	/**
	 * number of frequent and infrequent k-item-sets
	 */
	private Integer[] nrKItemSets;
	
	/**
	 * number of frequent k-item-sets
	 */
	private Integer[] nrFrequentKItemSets;
	
	/**
	 * number of CINDs
	 */
	private int nrCINDs = 0;
	
	/**
	 * average memory used
	 */
	private long avgMemoryUsed = -1;
	
	/**
	 * maximum memory used
	 */
	private long maxMemoryUsed = -1;
	
	
	/**
	 * Get singleton instance.
	 * 
	 * @return
	 */
	public static RuntimeMeasures getInstance() {
		return instance;
	}
	
	/**
	 * Singleton object.
	 */
	private RuntimeMeasures() {
		runtimeKItemSets = new Integer[20];
		nrFrequentKItemSets = new Integer[20];
		nrKItemSets = new Integer[20];
		
		Arrays.fill(runtimeKItemSets, -1);
		Arrays.fill(nrFrequentKItemSets, -1);
		Arrays.fill(nrKItemSets, -1);
	}
	
	
	public void printToStdOut(boolean humanReadable) {
		
		if (humanReadable) {

			
			System.out.println("time to read data: " + runtimeReadData);
			for (int i = 0; i < runtimeKItemSets.length; i++) {
				System.out.println("  time to create " + (i+1) + "-item-sets: "
						+ runtimeKItemSets[i] 
						+ " (" + nrFrequentKItemSets[i]
						+ " out of " + nrKItemSets[i]
						+ " item sets)");
			}
			System.out.println("time to generate rules: " + runtimeRuleGen);
			System.out.println("found " + nrCINDs + " CINDs");

			System.out.println("Laufzeit [ms]: " + runtimeOverAll);
			System.out.println("max. Speicherbedarf [KB]: " + maxMemoryUsed);
			System.out.println("avg. Speicherbedarf [KB]: " + avgMemoryUsed);
			
		}
			
		else {
			
//			System.out.format(Locale.ENGLISH, "%.5f, %.3f, %.4f, ", minSupport, minValidity, minTotalCompleteness); 
			System.out.format("%.5f; %.3f; %.4f; ", minSupport, minValidity, minTotalCompleteness); 
			System.out.format("%4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; ", (Object[]) nrKItemSets);
			System.out.format("%4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; ", (Object[]) nrFrequentKItemSets);
			System.out.format("%4d; ", nrCINDs);
			System.out.format("%5d; ", runtimeReadData);
			System.out.format("%5d; %5d; %5d; %5d; %5d; %5d; %5d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; %4d; ", (Object[]) runtimeKItemSets);
			System.out.format("%5d; ", runtimeRuleGen);
			System.out.format("%d; ", runtimeOverAll);
			System.out.format("%d; ", maxMemoryUsed);
			System.out.format("%d\n", avgMemoryUsed);
			
		}

	}
	
	
	/*
	 * setter methods
	 */
	
	public void setMinSupport(double minSupport) {
		this.minSupport = minSupport;
	}

	public void setMinValidity(double minValidity) {
		this.minValidity = minValidity;
	}

	public void setMinTotalCompleteness(double minTotalCompleteness) {
		this.minTotalCompleteness = minTotalCompleteness;
	}

	public void setRuntimeReadData(int runtime) {
		this.runtimeReadData = runtime;
	}

	public void setRuntimeKItemSets(int itemSetSize, Integer runtime) {
		this.runtimeKItemSets[itemSetSize] = runtime;
	}
	
	public void setRuntimeOverAll(int runtime) {
		this.runtimeOverAll = runtime;
	}

	public void setNrFrequentKItemSets(int itemSetSize, Integer nrFrequentKItemSets) {
		this.nrFrequentKItemSets[itemSetSize] = nrFrequentKItemSets;
	}

	public void setNrKItemSets(int itemSetSize, Integer nrKItemSets) {
		this.nrKItemSets[itemSetSize] = nrKItemSets;
	}
	
	public void setRuntimeRuleGen(Integer runtimeRuleGen) {
		this.runtimeRuleGen = runtimeRuleGen;
	}

	public void setNrCINDs(int nrCINDs) {
		this.nrCINDs = nrCINDs;
	}

	public void setAvgMemoryUsed(long avgMemoryUsed) {
		this.avgMemoryUsed = avgMemoryUsed;
	}

	public void setMaxMemoryUsed(long maxMemoryUsed) {
		this.maxMemoryUsed = maxMemoryUsed;
	}

	
}
