/**
 * 
 */
package de.unipotsdam.hpi.cind.associationRuleMining.util;

import java.util.*;

import junit.framework.*;

/**
 * @author jana.bauckmann
 *
 */
public class HashTreeNodeTest extends TestCase {

	private HashTreeNode tree;
	private ItemSetString itemset1, itemset2, itemset3, itemset4, itemset5,
			itemset6, itemset7, itemset8, itemset9;
	Basket basket;
	
    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
 
    	// add more item sets to get following tree
//    	interiorNode[afoo1] 
//    	         	interiorNode[bar1] 
//    	         		leaf: (afoo1, bar1, blub1), 
//    	         	interiorNode[bar2] 
//    	         		interiorNode[blub1] 
//    	         			leaf: (afoo1, bar2, blub1), 
//    	         		interiorNode[blub2] 
//    	         			leaf: (afoo1, bar2, blub2), 
//    	         		interiorNode[blub3] 
//    	         			leaf: (afoo1, bar2, blub3), 
//    	         		interiorNode[blub4] 
//    	         			leaf: (afoo1, bar2, blub4), 
//    	interiorNode[foo6] 
//    	         	leaf: (afoo6, bar7, blub7), (afoo6, bar7, blub6), (afoo6, bar6, blub5), 
        this.tree = new HashTreeNode(3);
    	
//        "foo1", "foo2", "bar1", "bar2", "blub1"
        
    	itemset1 = new ItemSetString("afoo1", "bar1", "blub1");
    	itemset2 = new ItemSetString("afoo1", "bar2", "blub1");
    	itemset3 = new ItemSetString("afoo1", "bar2", "blub2");
    	itemset4 = new ItemSetString("afoo1", "bar2", "blub3");
    	itemset5 = new ItemSetString("afoo1", "bar2", "blub4");
    	itemset6 = new ItemSetString("afoo6", "bar6", "blub5");
    	itemset7 = new ItemSetString("afoo6", "bar7", "blub6");
    	itemset8 = new ItemSetString("afoo6", "bar7", "blub7");
    	itemset9 = new ItemSetString("afoo1");
     
    	tree.addItemSet(itemset1);
    	tree.addItemSet(itemset2);
    	tree.addItemSet(itemset3);
    	tree.addItemSet(itemset4);
    	tree.addItemSet(itemset5);
    	tree.addItemSet(itemset6);
    	tree.addItemSet(itemset7);
    	tree.addItemSet(itemset8);
    	tree.addItemSet(itemset9);
    	
//    	System.out.println(tree.toString());
    	
    	// fill basket
    	// contains item set 1, 2, 8
    	ArrayList<String> items = new ArrayList<String>(
    			Arrays.asList("afoo1", "afoo2", "bar1", "bar2", "blub1", "afoo6", "bar7", "blub7"));
    	Collections.sort(items);
    	
    	basket = new Basket(items);
    	
    }
    
    /**
     * Constructor for HashTreeNodeTest.
     * @param arg0
     */
    public HashTreeNodeTest(String arg0) {
        super(arg0);
    }
	
    
    public void testHashTreeStructure() {
    	
    	// assert contains
    	assertTrue(tree.contains(itemset2));
    	
    	// do not add equal item sets
    	assertFalse(tree.addItemSet(itemset2));
    	
    	// add new item sets
    	assertTrue(tree.addItemSet(new ItemSetString("foo2", "bar1", "blub1")));
    	
    	// assert size of leaf nodes 
    	assertEquals(1, tree.sizeOfLeafContaining(itemset1));
    	assertEquals(3, tree.sizeOfLeafContaining(itemset8));
    	
    	// assert item set depth
    	assertEquals(2, tree.getDepthForItemSet(itemset1));
    	assertEquals(3, tree.getDepthForItemSet(itemset2));
    	assertEquals(1, tree.getDepthForItemSet(itemset8));
    	
    }
    
    public void testGetContainedItemSets() {

    	System.out.println(tree.toString());
    	
    	ArrayList<ItemSetString> containedItemSets = tree.getContainedItemSets(basket);
    	System.out.println(containedItemSets.toString());
    	
    	ArrayList<ItemSetString> expected = new ArrayList<ItemSetString>();
    	expected.add(itemset9);
    	expected.add(itemset1);
    	expected.add(itemset2);
    	expected.add(itemset8);
    	
    	assertEquals(expected, containedItemSets);
    	
    }
    
    
    public void testIncreaseSupportOfContainedItemSets() {
    	
    	assertTrue(tree.increaseSupportOfContainedItemSets(basket));
    	
    	assertEquals(1, itemset1.getFrequency());
    	assertEquals(1, itemset2.getFrequency());
    	assertEquals(1, itemset8.getFrequency());
    	assertEquals(1, itemset9.getFrequency());
    	assertEquals(0, itemset3.getFrequency());
    	assertEquals(0, itemset4.getFrequency());
    	assertEquals(0, itemset5.getFrequency());
    	assertEquals(0, itemset6.getFrequency());
    	assertEquals(0, itemset7.getFrequency());
    
    }
    
//    public void testIterator() {
//    	
//    	for (ItemSetString currentItemSet : tree) {
//			System.out.println(currentItemSet);
//		}
//    	System.out.println(tree.isEmpty());
//    	System.out.println(">>>>>>>>>>>>>");
//    	
//    	for (Iterator<ItemSetString> it = tree.iterator(); it.hasNext();) {
//			ItemSetString is = it.next();
//			System.out.println(is);
//			it.remove();
//		}
//    	System.out.println(tree.isEmpty());
//    	System.out.println(">>>>>>>>>>>>>");
//    	
//    	
//    	for (ItemSetString currentItemSet : tree) {
//			System.out.println(currentItemSet);
//		}
//    	
//    	System.out.println(tree.isEmpty());
//    }
//    
    
//    public void testLeafIterator() {
//    	
//    	for (Iterator<HashSet<ItemSetString>> it = tree.leafIterator(); it.hasNext();) {
//    		HashSet<ItemSetString> currentLeaf = it.next();
//			
//    		System.out.println("");
//    		for (ItemSetString currentItemSet : currentLeaf) {
//				System.out.println(currentItemSet.toString());
//			}
//		}
//    	
//    	
//    	System.out.println(">>>>>>>>>>>>>");
//    	
//    	ItemSetString toDelete = new ItemSetString("foo1", "bar2", "blub1");
//    	
//    	for (Iterator<ItemSetString> it = tree.iterator(); it.hasNext();) {
//			ItemSetString is = it.next();
//			if (is.equals(toDelete) ) 
//				it.remove();
//		}
//    	
//    	for (Iterator<HashSet<ItemSetString>> it = tree.leafIterator(); it.hasNext();) {
//    		HashSet<ItemSetString> currentLeaf = it.next();
//			
//    		System.out.println("");
//    		for (ItemSetString currentItemSet : currentLeaf) {
//				System.out.println(currentItemSet.toString());
//			}
//		}
//    	
//    }
    
}
